from bioservices.quickgo import QuickGO
import pytest

@pytest.fixture
def quickgo():
    return QuickGO(verbose=False, cache=False)

def test_annotation_taxonid(quickgo):
    res = quickgo.Annotation(taxonId='9606')

def test_annotation_aspect(quickgo):
    res = quickgo.Annotation(taxonId="9606", limit=100, aspect="F")

def test_annotation_geneProductType(quickgo):
    res = quickgo.Annotation(taxonId="9606", limit=100, geneProductType="protein")

def test_annotations_from_goid(quickgo):
    df = quickgo.Annotation_from_goid(goId="GO:0003824,GO:0003677", taxonId="9606" )
    assert len(df)

def test_goterms(quickgo):
    res = quickgo.goterms(max_number_of_pages=5)


def test_Term(quickgo):
    res = quickgo.Terms("GO:0003824,GO:0003677")
    assert len(res) == 2



from bioservices import Miriam
import pytest
import os

pytestmark = pytest.mark.skipif( "TRAVIS_PYTHON_VERSION" in os.environ,
         reason="On travis")



@pytest.fixture
def miriam():
    return Miriam(verbose=False)


def test_checkRegExp(miriam):
    assert miriam.checkRegExp("1P43403", "uniprot") == False
    assert miriam.checkRegExp("P43403", "uniprot") == True

def test_convertURL(miriam):
    assert miriam.convertURL("http://identifiers.org/ec-code/1.1.1.1") == 'urn:miriam:ec-code:1.1.1.1'

def test_convertURLs(miriam):
    miriam.convertURLs(['http://identifiers.org/pubmed/16333295', 'http://identifiers.org/ec-code/1.1.1.1'])

def test_convertURN(miriam):
    res = miriam.convertURN('urn:miriam:ec-code:1.1.1.1')
    assert res == 'http://identifiers.org/ec-code/1.1.1.1'

def test_convertURNs(miriam):
    res = miriam.convertURNs(['urn:miriam:ec-code:1.1.1.1'])
    assert res == ['http://identifiers.org/ec-code/1.1.1.1']

def test_getDataResources(miriam):
    miriam.getDataResources("uniprot")

def test_getDataTypeDef(miriam):
    miriam.getDataTypeDef("uniprot")

def test_getDataTypePattern(miriam):
    miriam.getDataTypePattern("uniprot")

def test_getDataTypeSynonyms(miriam):
    assert len(miriam.getDataTypeSynonyms("uniprot"))>0

def test_getDataTypeURI(miriam):
    assert miriam.getDataTypeURI("uniprot") == 'urn:miriam:uniprot'

def test_getDataTypeURIs(miriam):
    miriam.getDataTypeURIs("uniprot")
    ['urn:miriam:uniprot', 'urn:lsid:uniprot.org:uniprot', 'urn:lsid:uniprot.org', 'http://www.uniprot.org/']

def test_getDataTypesId(miriam):
    miriam.getDataTypesId()

def test_getDataTypesName(miriam):
    miriam.getDataTypesName()

def test_getJavaLibraryVersion(miriam):
    miriam.getJavaLibraryVersion()

def test_getLocation(miriam):
    miriam.getLocation("UniProt", "MIR:00100005")
    miriam.getLocations("urn:miriam:obo.go:GO%3A0045202")


def test_getLocations(miriam ):
    miriam.getLocations("UniProt","P62158")
    #>>> m.serv.getLocations("urn:miriam:obo.go:GO%3A0045202")

def test_getLocationsWithToken(miriam):
    miriam.getLocationsWithToken("uniprot", "P43403")

def test_getMiriamURI(miriam):
    res = miriam.getMiriamURI("http://www.ebi.ac.uk/chebi/#CHEBI:17891")
    assert res == 'urn:miriam:chebi:CHEBI%3A17891'

def test_getName(miriam):
    miriam.getName('uniprot')

def test_getNames(miriam):
    miriam.getNames('uniprot')

def test_getOfficialDataTypeURI(miriam):
    miriam.getOfficialDataTypeURI("chEBI")
    #'urn:miriam:chebi'

def test_getResourceInfo(miriam):
    res = miriam.getResourceInfo("MIR:00100005")
    assert res == 'MIRIAM Resources (data collection)'

def test_getResourceInstitution(miriam):
    res = miriam.getResourceInstitution("MIR:00100005")
    assert res == "European Bioinformatics Institute, Hinxton, Cambridge"

def test_getResourceLocation(miriam):
    res = miriam.getResourceLocation("MIR:00100005")
    assert res == "UK"

def test_getServicesInfo(miriam):
    miriam.getServicesInfo()

def test_getServicesVersion(miriam):
    miriam.getServicesVersion()

def test_getURI(miriam):
    res = miriam.getURI("UniProt", "P62158")
    assert res == 'urn:miriam:uniprot:P62158'

def test_getURIs(miriam):
    res = miriam.getURIs("UniProt", "P62158")

def test_isDeprecated(miriam):
    miriam.isDeprecated("urn:miriam:uniprot")




from bioservices import NCBIblast
import pytest
import os

skiptravis = pytest.mark.skipif( "TRAVIS_PYTHON_VERSION" in os.environ,
    reason="too slow for travis")


@pytest.fixture
def ncbi():
    return NCBIblast()


def test_param(ncbi):
    ncbi.getParameters()
    assert(len(ncbi.parameters)>0)
    assert(len(ncbi.parameters)>0)


def test_paramdetails(ncbi):
    names = ncbi.parametersDetails("matrix") 
    try:
        names = ncbi.parametersDetails("matrixddddd") 
        assert False
    except:
        assert True

@skiptravis
def test_run(ncbi):

    try:
        ncbi.jobid = ncbi.run(program="blastp", sequence=ncbi._sequence_example,
            stype="protein", database="uniprotkb")
        assert False # missing email argument
    except:
        assert True
    ncbi.jobid = ncbi.run(program="blastp", sequence=ncbi._sequence_example,
        stype="protein", database="uniprotkb", email="cokelaer@ebi.ac.uk",
        matrix="BLOSUM45")
    res = ncbi.getResult(ncbi.jobid, "out")

    res = ncbi.getResultTypes(ncbi.jobid)

def test_attributse(ncbi):
    ncbi.databases



from bioservices import seqret
import pytest



fasta = """MDAPRQVVNFGPGPAKLPHSVLLEIQKELLDYKGVGISVLEMSHRSSDFAKIINNTENLVRELLAVPDNYKVIFLQGGGCGQFSAVPLNLIGLKAGRCADYVVTGAWSAKAAEEAKKFGTINIVHPKLGSYTKIPDPSTWNLNPDASYVYYCANETVHGVEFDFIPDVKGAVLVCDMSSNFLSKPVDVSKFGVIFAGAQKNVGSAGVTVVIVRDDLLGFALRECPSVLEYKVQAGNSSLYNTPPCFSIYVMGLVLEWIKNNGGAAAMEKLSSIKSQTIYEIIDNSQGFYVCPVEPQNRSKMNIPFRIGNAKGDDALEKRFLDKALELNMLSLKGHRSVGGIRASLYNAVTIEDVQKLAAFMKKFLEMHQL"""


def test_readseq():

    s = seqret.Seqret()
    jobid = s.run(**{"email":"cokelaer@test.co.uk", "title":"test", "sequence":fasta,
        "inputformat":"raw", "outputformat":"fasta", "stype":"protein"})

    import time
    count = 0
    while s.get_status(jobid) != "FINISHED" and count<20:
        count += 1
        time.sleep(1)
        print(count)

    newfasta = s.get_result(s._jobid)
    if newfasta:
        newfasta = newfasta.replace("\n", "")
        assert "DVQKLAAFMKKFLEMHQL" in newfasta
    else:
        pass # unfortunate but with slow connection, this may end up here 



from bioservices.chembl import ChEMBL
import pytest
import os

skiptravis = pytest.mark.skipif( "TRAVIS_PYTHON_VERSION" in os.environ,
     reason="On travis")


SMILE = "CC(=O)Oc1ccccc1C(=O)O"
INCHIKEY = "BSYNRYMUTXBXSQ-UHFFFAOYSA-N"


@pytest.fixture
def chembl():
    c = ChEMBL(verbose=False, cache=False)
    c.TIMEOUT = 60
    c.default_extension = "xml"
    try:
        c.default_extension = "xmlf"
        assert False
    except:
        assert True
    return c


def test_filters(chembl):
    # test without filters
    res = chembl.get_molecule(limit=20)
    assert len(res) == 20

    # test with one string
    res1 = chembl.get_molecule(limit=20,
        filters='molecule_properties__mw_freebase__lte=300')
    assert len(res1) == 20

    # test with one filter in a list
    res2 = chembl.get_molecule(limit=20,
        filters=['molecule_properties__mw_freebase__lte=300'])
    assert len(res2) == 20

    assert res1 == res2

    # test with several filters in a list

    res = chembl.get_molecule(limit=20,
        filters=[
            'molecule_properties__mw_freebase__lte=300',
            'pref_name__iendswith=nib'])
    assert len(res)
    for this in res:
        assert 'nib' in this['pref_name'].lower()
        assert float(this['molecule_properties']['mw_freebase'])< 300


def test_assay_filter(chembl):
    # test get_assay using a filter:
    res = chembl.get_assay(filters=['assay_type__in=F'])
    assert set([this['assay_type'] for this in res]) == set(['F'])



# ACTIVITIES

def test_activity(chembl):
    res = chembl.get_activity()
    res = chembl.get_activity(['31863','31864'])
    res = chembl.get_activity(['31863',31864])
    assert len(res) == 2

    # here one will not be found (-1)
    res = chembl.get_activity(['31863','-1'])
    assert len(res) == 1
    assert chembl.not_found == ['-1']


# Test ALL resources get_<RESOURCE>

@pytest.mark.parametrize('name', [
    'assay', 'chembl_id_lookup', 'cell_line', 'compound_record', 'biotherapeutic',
    'chembl_id_lookup', 'binding_site', 'organism', 'compound_structural_alert',
    'document', 'document_similarity', 'document_term', 'drug',
    'drug_indication', 'go_slim', 'mechanism', 'metabolism', 'molecule_form',
    'protein_class', 'source', 'target', 'target_component',
    'target_prediction', 'tissue', 'target_relation', 'xref_source']
    )
def test_resource(chembl, name):
    res = getattr(chembl, "get_" + name)()
    assert len(res)

# This one has no S at the end ? error in the API or assuming it is an acronym
# therefore without S
def test_ATC(chembl):
    res = chembl.get_ATC()
    assert res[0]['who_name']


# SEARCHES ------------------

@skiptravis
def test_search_protein_class(chembl):
    res1715 = chembl.get_protein_class(1715)
    # no good example. This returns noting but at least calls the method
    chembl.search_protein_class('CAMK')

def test_search_target(chembl):
     res = chembl.search_target('cyclin')
     assert len(res['targets'])

def test_search_molecule(chembl):
    res = chembl.search_molecule('aspirin')
    assert len(res['molecules'])

def test_search_assay(chembl):
    res = chembl.search_assay("aspirin")
    assert len(res['assays'])

def test_search_activity(chembl):
    res = chembl.search_activity("GATES")
    assert len(res['activities'])

def test_search_document(chembl):
    res =  chembl.search_document("cytokine")['documents']
    assert len(res)

def test_search_chembl_id(chembl):
    res = chembl.search_chembl_id_lookup('morphine')['chembl_id_lookups']
    assert 'chembl_id' in res[0]


# Test different ways of using the get_<method>
def test_molecule(chembl):

    res = chembl.get_molecule('CHEMBL25')
    assert res['molecule_chembl_id'] == 'CHEMBL25'

    # list
    res = chembl.get_molecule(['CHEMBL25', 'CHEMBL100'])
    assert len(res) == 2

    # list with a wrong ID
    res = chembl.get_molecule(['CHEMBL25', 'CHEMBL00'])
    assert len(res) == 1
    assert chembl.not_found == ['CHEMBL00']



def test_substructure(chembl):

    res = chembl.get_substructure(SMILE)
    assert res[0]['molecule_structures']['canonical_smiles'] == SMILE

    # search by chembl I
    res = chembl.get_similarity("CHEMBL25")
    assert len(res) == 1

    # search by chembl I
    res = chembl.get_similarity(INCHIKEY)
    assert res


def test_similarity(chembl):

    # search by smiles
    res = chembl.get_similarity(SMILE)
    assert res

    # search by chembl I
    res = chembl.get_similarity("CHEMBL25")
    assert res

    # search by chembl I
    res = chembl.get_similarity(INCHIKEY)
    assert res


# others ---------------------------------------------

def test_get_approved_drugs(chembl):
    res = chembl.get_approved_drugs(maxdrugs=20)


def test_image(chembl):
    res = chembl.get_image("CHEMBL25", view=False)
    os.remove("CHEMBL25.png")
    res = chembl.get_image("CHEMBL25", view=False, format='svg')
    os.remove("CHEMBL25.svg")


def test_get_status(chembl):
    res = chembl.get_status()
    assert 'activities' in res


def test_get_status_resources(chembl):
    res = chembl.get_status_resources()
    assert 'activity' in res


def test_int_str_request(chembl):
     assert chembl.get_mechanism(13) == chembl.get_mechanism('13')


def test_ordering(chembl):

    res = chembl.get_molecule(limit=100)

    # simple key
    chembl.order_by(res,'molecule_chembl_id')

    # double key
    data1 = [x['molecule_properties']['acd_logd'] for x in chembl.order_by(res, 'molecule_properties__acd_logd', ascending=True)]
    assert data1[0] < data1[1]

    data2 = [x['molecule_properties']['acd_logd'] for x in chembl.order_by(res,'molecule_properties__acd_logd', ascending=False)]
    assert data2[0] > data2[1]

    # triple key: FIXME no exqample

    # more:
    try:
        chembl.order_by(res, 'test__test__test__test')
        assert False
    except:
        assert True


# very slow and probably useless now. 
# the test was written to check an unclear behaviou
def __test_limit(chembl):
    # Check that the limi
    # See notes in _get_data in ChEMBL.
    # if limit > 1000 and offset >0 ChEMBL REST API reset the limit to 1000-10,
    # which is not expected. 
    offset = 10
    res = chembl.get_mechanism(limit=6000, offset=offset)
    assert len(res) == chembl.page_meta['total_count'] - offset

# very very slow 
def __test_compounds2accession(chembl):
    res = chembl.compounds2accession('CHEMBL4')



from bioservices import RNASEQ_EBI
import os
import pytest

# This services is deprecated
'''
skiptravis = pytest.mark.skipif( "TRAVIS_PYTHON_VERSION" in os.environ,
     reason="in maintenance")


@skiptravis
@pytest.fixture
def rnaseq():
    r = RNASEQ_EBI(cache=False)
    assert 'homo_sapiens' in r.organisms
    return r


@skiptravis
def test1(rnaseq):
    rnaseq.get_run_by_organism("homo_sapiens", "tsv")
    rnaseq.get_run_by_organism("homo_sapiens", "json")
    rnaseq.get_run_by_organism("homo_sapiens",condition="central nervous system")


@skiptravis
def test2(rnaseq):
    rnaseq.get_run_by_study("SRP033494", mapping_quality=90, frmt='tsv')


@skiptravis
def test3(rnaseq):
    res = rnaseq.get_study("SRP033494", "tsv")
    res = rnaseq.get_study("SRP033494", frmt="json")
    assert res[0]['STUDY_ID'] == "SRP033494"


@skiptravis
def test4(rnaseq):
    try:
        import pandas
        res = rnaseq.get_studies_by_organism("arabidopsis_thaliana", frmt='tsv')
        studies = res['STUDY_ID'].values
    except:
        res = rnaseq.get_studies_by_organism("arabidopsis_thaliana", frmt='tsv')
        studies = [x[0] for x in res[1:]]


@skiptravis
def test5(rnaseq):
    rnaseq.get_sample_attribute_per_run("SRR805786")
    rnaseq.get_sample_attribute_per_run("SRR805786", frmt='tsv')

    rnaseq.get_sample_attribute_per_study("SRP020492")
    rnaseq.get_sample_attribute_per_study("SRP020492", frmt='tsv')

    rnaseq.get_sample_attribute_per_study("SRP020492")
    rnaseq.get_sample_attribute_per_study("SRP020492", frmt='tsv')


@skiptravis
def test_get_run(rnaseq):
    res = rnaseq.get_run("SRR1042759")
    assert res[0]['RUN_IDS'] == "SRR1042759"
'''
